@@ -2,7 +2,7 @@
 // strand.hpp
 // ~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -49,20 +49,20 @@ public:
   /// Construct a strand for the specified executor.
   template <typename Executor1>
   explicit strand(const Executor1& e,
-      typename constraint<
-        conditional<
+      constraint_t<
+        conditional_t<
           !is_same<Executor1, strand>::value,
           is_convertible<Executor1, Executor>,
           false_type
-        >::type::value
-      >::type = 0)
+        >::value
+      > = 0)
     : executor_(e),
       impl_(strand::create_implementation(executor_))
   {
   }
 
   /// Copy constructor.
-  strand(const strand& other) BOOST_ASIO_NOEXCEPT
+  strand(const strand& other) noexcept
     : executor_(other.executor_),
       impl_(other.impl_)
   {
@@ -75,14 +75,14 @@ public:
    */
   template <class OtherExecutor>
   strand(
-      const strand<OtherExecutor>& other) BOOST_ASIO_NOEXCEPT
+      const strand<OtherExecutor>& other) noexcept
     : executor_(other.executor_),
       impl_(other.impl_)
   {
   }
 
   /// Assignment operator.
-  strand& operator=(const strand& other) BOOST_ASIO_NOEXCEPT
+  strand& operator=(const strand& other) noexcept
   {
     executor_ = other.executor_;
     impl_ = other.impl_;
@@ -96,18 +96,17 @@ public:
    */
   template <class OtherExecutor>
   strand& operator=(
-      const strand<OtherExecutor>& other) BOOST_ASIO_NOEXCEPT
+      const strand<OtherExecutor>& other) noexcept
   {
     executor_ = other.executor_;
     impl_ = other.impl_;
     return *this;
   }
 
-#if defined(BOOST_ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
   /// Move constructor.
-  strand(strand&& other) BOOST_ASIO_NOEXCEPT
-    : executor_(BOOST_ASIO_MOVE_CAST(Executor)(other.executor_)),
-      impl_(BOOST_ASIO_MOVE_CAST(implementation_type)(other.impl_))
+  strand(strand&& other) noexcept
+    : executor_(static_cast<Executor&&>(other.executor_)),
+      impl_(static_cast<implementation_type&&>(other.impl_))
   {
   }
 
@@ -117,17 +116,17 @@ public:
    * to @c Executor.
    */
   template <class OtherExecutor>
-  strand(strand<OtherExecutor>&& other) BOOST_ASIO_NOEXCEPT
-    : executor_(BOOST_ASIO_MOVE_CAST(OtherExecutor)(other.executor_)),
-      impl_(BOOST_ASIO_MOVE_CAST(implementation_type)(other.impl_))
+  strand(strand<OtherExecutor>&& other) noexcept
+    : executor_(static_cast<OtherExecutor&&>(other.executor_)),
+      impl_(static_cast<implementation_type&&>(other.impl_))
   {
   }
 
   /// Move assignment operator.
-  strand& operator=(strand&& other) BOOST_ASIO_NOEXCEPT
+  strand& operator=(strand&& other) noexcept
   {
-    executor_ = BOOST_ASIO_MOVE_CAST(Executor)(other.executor_);
-    impl_ = BOOST_ASIO_MOVE_CAST(implementation_type)(other.impl_);
+    executor_ = static_cast<Executor&&>(other.executor_);
+    impl_ = static_cast<implementation_type&&>(other.impl_);
     return *this;
   }
 
@@ -137,21 +136,20 @@ public:
    * convertible to @c Executor.
    */
   template <class OtherExecutor>
-  strand& operator=(strand<OtherExecutor>&& other) BOOST_ASIO_NOEXCEPT
+  strand& operator=(strand<OtherExecutor>&& other) noexcept
   {
-    executor_ = BOOST_ASIO_MOVE_CAST(OtherExecutor)(other.executor_);
-    impl_ = BOOST_ASIO_MOVE_CAST(implementation_type)(other.impl_);
+    executor_ = static_cast<OtherExecutor&&>(other.executor_);
+    impl_ = static_cast<implementation_type&&>(other.impl_);
     return *this;
   }
-#endif // defined(BOOST_ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
 
   /// Destructor.
-  ~strand() BOOST_ASIO_NOEXCEPT
+  ~strand() noexcept
   {
   }
 
   /// Obtain the underlying executor.
-  inner_executor_type get_inner_executor() const BOOST_ASIO_NOEXCEPT
+  inner_executor_type get_inner_executor() const noexcept
   {
     return executor_;
   }
@@ -168,16 +166,15 @@ public:
    *   ... @endcode
    */
   template <typename Property>
-  typename constraint<
+  constraint_t<
     can_query<const Executor&, Property>::value,
-    typename conditional<
+    conditional_t<
       is_convertible<Property, execution::blocking_t>::value,
       execution::blocking_t,
-      typename query_result<const Executor&, Property>::type
-    >::type
-  >::type query(const Property& p) const
-    BOOST_ASIO_NOEXCEPT_IF((
-      is_nothrow_query<const Executor&, Property>::value))
+      query_result_t<const Executor&, Property>
+    >
+  > query(const Property& p) const
+    noexcept(is_nothrow_query<const Executor&, Property>::value)
   {
     return this->query_helper(
         is_convertible<Property, execution::blocking_t>(), p);
@@ -194,19 +191,15 @@ public:
    *     boost::asio::execution::blocking.never); @endcode
    */
   template <typename Property>
-  typename constraint<
+  constraint_t<
     can_require<const Executor&, Property>::value
       && !is_convertible<Property, execution::blocking_t::always_t>::value,
-    strand<typename decay<
-      typename require_result<const Executor&, Property>::type
-    >::type>
-  >::type require(const Property& p) const
-    BOOST_ASIO_NOEXCEPT_IF((
-      is_nothrow_require<const Executor&, Property>::value))
+    strand<decay_t<require_result_t<const Executor&, Property>>>
+  > require(const Property& p) const
+    noexcept(is_nothrow_require<const Executor&, Property>::value)
   {
-    return strand<typename decay<
-      typename require_result<const Executor&, Property>::type
-        >::type>(boost::asio::require(executor_, p), impl_);
+    return strand<decay_t<require_result_t<const Executor&, Property>>>(
+        boost::asio::require(executor_, p), impl_);
   }
 
   /// Forward a preference to the underlying executor.
@@ -220,24 +213,20 @@ public:
    *     boost::asio::execution::blocking.never); @endcode
    */
   template <typename Property>
-  typename constraint<
+  constraint_t<
     can_prefer<const Executor&, Property>::value
       && !is_convertible<Property, execution::blocking_t::always_t>::value,
-    strand<typename decay<
-      typename prefer_result<const Executor&, Property>::type
-    >::type>
-  >::type prefer(const Property& p) const
-    BOOST_ASIO_NOEXCEPT_IF((
-      is_nothrow_prefer<const Executor&, Property>::value))
+    strand<decay_t<prefer_result_t<const Executor&, Property>>>
+  > prefer(const Property& p) const
+    noexcept(is_nothrow_prefer<const Executor&, Property>::value)
   {
-    return strand<typename decay<
-      typename prefer_result<const Executor&, Property>::type
-        >::type>(boost::asio::prefer(executor_, p), impl_);
+    return strand<decay_t<prefer_result_t<const Executor&, Property>>>(
+        boost::asio::prefer(executor_, p), impl_);
   }
 
 #if !defined(BOOST_ASIO_NO_TS_EXECUTORS)
   /// Obtain the underlying execution context.
-  execution_context& context() const BOOST_ASIO_NOEXCEPT
+  execution_context& context() const noexcept
   {
     return executor_.context();
   }
@@ -246,7 +235,7 @@ public:
   /**
    * The strand delegates this call to its underlying executor.
    */
-  void on_work_started() const BOOST_ASIO_NOEXCEPT
+  void on_work_started() const noexcept
   {
     executor_.on_work_started();
   }
@@ -255,7 +244,7 @@ public:
   /**
    * The strand delegates this call to its underlying executor.
    */
-  void on_work_finished() const BOOST_ASIO_NOEXCEPT
+  void on_work_finished() const noexcept
   {
     executor_.on_work_finished();
   }
@@ -263,13 +252,6 @@ public:
 
   /// Request the strand to invoke the given function object.
   /**
-   * Do not call this function directly. It is intended for use with the
-   * execution::execute customisation point.
-   *
-   * For example:
-   * @code boost::asio::strand<my_executor_type> ex = ...;
-   * execution::execute(ex, my_function_object); @endcode
-   *
    * This function is used to ask the strand to execute the given function
    * object on its underlying executor. The function object will be executed
    * according to the properties of the underlying executor.
@@ -279,13 +261,13 @@ public:
    * function object must be: @code void function(); @endcode
    */
   template <typename Function>
-  typename constraint<
-    execution::can_execute<const Executor&, Function>::value,
+  constraint_t<
+    traits::execute_member<const Executor&, Function>::is_valid,
     void
-  >::type execute(BOOST_ASIO_MOVE_ARG(Function) f) const
+  > execute(Function&& f) const
   {
     detail::strand_executor_service::execute(impl_,
-        executor_, BOOST_ASIO_MOVE_CAST(Function)(f));
+        executor_, static_cast<Function&&>(f));
   }
 
 #if !defined(BOOST_ASIO_NO_TS_EXECUTORS)
@@ -305,10 +287,10 @@ public:
    * internal storage needed for function invocation.
    */
   template <typename Function, typename Allocator>
-  void dispatch(BOOST_ASIO_MOVE_ARG(Function) f, const Allocator& a) const
+  void dispatch(Function&& f, const Allocator& a) const
   {
     detail::strand_executor_service::dispatch(impl_,
-        executor_, BOOST_ASIO_MOVE_CAST(Function)(f), a);
+        executor_, static_cast<Function&&>(f), a);
   }
 
   /// Request the strand to invoke the given function object.
@@ -325,10 +307,10 @@ public:
    * internal storage needed for function invocation.
    */
   template <typename Function, typename Allocator>
-  void post(BOOST_ASIO_MOVE_ARG(Function) f, const Allocator& a) const
+  void post(Function&& f, const Allocator& a) const
   {
     detail::strand_executor_service::post(impl_,
-        executor_, BOOST_ASIO_MOVE_CAST(Function)(f), a);
+        executor_, static_cast<Function&&>(f), a);
   }
 
   /// Request the strand to invoke the given function object.
@@ -345,10 +327,10 @@ public:
    * internal storage needed for function invocation.
    */
   template <typename Function, typename Allocator>
-  void defer(BOOST_ASIO_MOVE_ARG(Function) f, const Allocator& a) const
+  void defer(Function&& f, const Allocator& a) const
   {
     detail::strand_executor_service::defer(impl_,
-        executor_, BOOST_ASIO_MOVE_CAST(Function)(f), a);
+        executor_, static_cast<Function&&>(f), a);
   }
 #endif // !defined(BOOST_ASIO_NO_TS_EXECUTORS)
 
@@ -358,7 +340,7 @@ public:
    * submitted to the strand using post(), dispatch() or defer(). Otherwise
    * returns @c false.
    */
-  bool running_in_this_thread() const BOOST_ASIO_NOEXCEPT
+  bool running_in_this_thread() const noexcept
   {
     return detail::strand_executor_service::running_in_this_thread(impl_);
   }
@@ -368,7 +350,7 @@ public:
    * Two strands are equal if they refer to the same ordered, non-concurrent
    * state.
    */
-  friend bool operator==(const strand& a, const strand& b) BOOST_ASIO_NOEXCEPT
+  friend bool operator==(const strand& a, const strand& b) noexcept
   {
     return a.impl_ == b.impl_;
   }
@@ -378,7 +360,7 @@ public:
    * Two strands are equal if they refer to the same ordered, non-concurrent
    * state.
    */
-  friend bool operator!=(const strand& a, const strand& b) BOOST_ASIO_NOEXCEPT
+  friend bool operator!=(const strand& a, const strand& b) noexcept
   {
     return a.impl_ != b.impl_;
   }
@@ -391,9 +373,9 @@ private:
 
   template <typename InnerExecutor>
   static implementation_type create_implementation(const InnerExecutor& ex,
-      typename constraint<
+      constraint_t<
         can_query<InnerExecutor, execution::context_t>::value
-      >::type = 0)
+      > = 0)
   {
     return use_service<detail::strand_executor_service>(
         boost::asio::query(ex, execution::context)).create_implementation();
@@ -401,9 +383,9 @@ private:
 
   template <typename InnerExecutor>
   static implementation_type create_implementation(const InnerExecutor& ex,
-      typename constraint<
+      constraint_t<
         !can_query<InnerExecutor, execution::context_t>::value
-      >::type = 0)
+      > = 0)
   {
     return use_service<detail::strand_executor_service>(
         ex.context()).create_implementation();
@@ -416,7 +398,7 @@ private:
   }
 
   template <typename Property>
-  typename query_result<const Executor&, Property>::type query_helper(
+  query_result_t<const Executor&, Property> query_helper(
       false_type, const Property& property) const
   {
     return boost::asio::query(executor_, property);
@@ -449,9 +431,9 @@ private:
  */
 template <typename Executor>
 inline strand<Executor> make_strand(const Executor& ex,
-    typename constraint<
+    constraint_t<
       is_executor<Executor>::value || execution::is_executor<Executor>::value
-    >::type = 0)
+    > = 0)
 {
   return strand<Executor>(ex);
 }
@@ -466,9 +448,9 @@ inline strand<Executor> make_strand(const Executor& ex,
 template <typename ExecutionContext>
 inline strand<typename ExecutionContext::executor_type>
 make_strand(ExecutionContext& ctx,
-    typename constraint<
+    constraint_t<
       is_convertible<ExecutionContext&, execution_context&>::value
-    >::type = 0)
+    > = 0)
 {
   return strand<typename ExecutionContext::executor_type>(ctx.get_executor());
 }
@@ -482,10 +464,10 @@ namespace traits {
 #if !defined(BOOST_ASIO_HAS_DEDUCED_EQUALITY_COMPARABLE_TRAIT)
 
 template <typename Executor>
-struct equality_comparable<strand<Executor> >
+struct equality_comparable<strand<Executor>>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
 };
 
 #endif // !defined(BOOST_ASIO_HAS_DEDUCED_EQUALITY_COMPARABLE_TRAIT)
@@ -494,12 +476,12 @@ struct equality_comparable<strand<Executor> >
 
 template <typename Executor, typename Function>
 struct execute_member<strand<Executor>, Function,
-    typename enable_if<
-      execution::can_execute<const Executor&, Function>::value
-    >::type>
+    enable_if_t<
+      traits::execute_member<const Executor&, Function>::is_valid
+    >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef void result_type;
 };
 
@@ -509,17 +491,16 @@ struct execute_member<strand<Executor>, Function,
 
 template <typename Executor, typename Property>
 struct query_member<strand<Executor>, Property,
-    typename enable_if<
+    enable_if_t<
       can_query<const Executor&, Property>::value
-    >::type>
+    >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept =
-      (is_nothrow_query<Executor, Property>::value));
-  typedef typename conditional<
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept =
+    is_nothrow_query<Executor, Property>::value;
+  typedef conditional_t<
     is_convertible<Property, execution::blocking_t>::value,
-      execution::blocking_t, typename query_result<Executor, Property>::type
-        >::type result_type;
+      execution::blocking_t, query_result_t<Executor, Property>> result_type;
 };
 
 #endif // !defined(BOOST_ASIO_HAS_DEDUCED_QUERY_MEMBER_TRAIT)
@@ -528,17 +509,15 @@ struct query_member<strand<Executor>, Property,
 
 template <typename Executor, typename Property>
 struct require_member<strand<Executor>, Property,
-    typename enable_if<
+    enable_if_t<
       can_require<const Executor&, Property>::value
         && !is_convertible<Property, execution::blocking_t::always_t>::value
-    >::type>
+    >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept =
-      (is_nothrow_require<Executor, Property>::value));
-  typedef strand<typename decay<
-    typename require_result<Executor, Property>::type
-      >::type> result_type;
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept =
+    is_nothrow_require<Executor, Property>::value;
+  typedef strand<decay_t<require_result_t<Executor, Property>>> result_type;
 };
 
 #endif // !defined(BOOST_ASIO_HAS_DEDUCED_REQUIRE_MEMBER_TRAIT)
@@ -547,17 +526,15 @@ struct require_member<strand<Executor>, Property,
 
 template <typename Executor, typename Property>
 struct prefer_member<strand<Executor>, Property,
-    typename enable_if<
+    enable_if_t<
       can_prefer<const Executor&, Property>::value
         && !is_convertible<Property, execution::blocking_t::always_t>::value
-    >::type>
+    >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept =
-      (is_nothrow_prefer<Executor, Property>::value));
-  typedef strand<typename decay<
-    typename prefer_result<Executor, Property>::type
-      >::type> result_type;
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept =
+    is_nothrow_prefer<Executor, Property>::value;
+  typedef strand<decay_t<prefer_result_t<Executor, Property>>> result_type;
 };
 
 #endif // !defined(BOOST_ASIO_HAS_DEDUCED_PREFER_MEMBER_TRAIT)
