@@ -1,7 +1,7 @@
 //  filesystem path_traits.hpp  --------------------------------------------------------//
 
 //  Copyright Beman Dawes 2009
-//  Copyright Andrey Semashev 2022
+//  Copyright Andrey Semashev 2022-2024
 
 //  Distributed under the Boost Software License, Version 1.0.
 //  See http://www.boost.org/LICENSE_1_0.txt
@@ -18,21 +18,19 @@
 #include <locale>
 #include <string>
 #include <iterator>
+#include <type_traits>
 #if !defined(BOOST_NO_CXX17_HDR_STRING_VIEW)
 #include <string_view>
 #endif
 #include <boost/assert.hpp>
 #include <boost/system/error_category.hpp>
 #include <boost/iterator/is_iterator.hpp>
-#include <boost/type_traits/declval.hpp>
-#include <boost/type_traits/remove_cv.hpp>
-#include <boost/type_traits/integral_constant.hpp>
-#include <boost/type_traits/conjunction.hpp>
+#include <boost/filesystem/detail/type_traits/negation.hpp>
+#include <boost/filesystem/detail/type_traits/conjunction.hpp>
 #if defined(BOOST_FILESYSTEM_DETAIL_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
-#include <boost/type_traits/disjunction.hpp>
-#include <boost/core/enable_if.hpp>
+#include <boost/filesystem/detail/type_traits/disjunction.hpp>
 #endif
-#if !defined(BOOST_FILESYSTEM_NO_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
+#if defined(BOOST_FILESYSTEM_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
 #include <vector>
 #include <list>
 #endif
@@ -49,8 +47,9 @@ template< typename, typename, typename > class basic_string;
 
 namespace filesystem {
 
-BOOST_FILESYSTEM_DECL system::error_category const& codecvt_error_category() BOOST_NOEXCEPT;
+BOOST_FILESYSTEM_DECL system::error_category const& codecvt_error_category() noexcept;
 
+class path;
 class directory_entry;
 
 namespace detail {
@@ -256,7 +255,7 @@ struct path_source_traits< boost::basic_string_view< wchar_t, std::char_traits<
     static BOOST_CONSTEXPR_OR_CONST bool is_native = false;
 };
 
-#if !defined(BOOST_FILESYSTEM_NO_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
+#if defined(BOOST_FILESYSTEM_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
 template< >
 struct
 BOOST_FILESYSTEM_DETAIL_DEPRECATED("Boost.Filesystem path construction/assignment/appending from containers is deprecated, use strings or iterators instead.")
@@ -298,7 +297,7 @@ path_source_traits< std::list< wchar_t > >
     typedef wchar_t char_type;
     static BOOST_CONSTEXPR_OR_CONST bool is_native = false;
 };
-#endif // !defined(BOOST_FILESYSTEM_NO_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
+#endif // defined(BOOST_FILESYSTEM_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
 
 template< >
 struct path_source_traits< directory_entry >
@@ -315,13 +314,13 @@ struct path_source_traits< directory_entry >
 //! The trait tests if the type is a known path Source tag
 template< typename Tag >
 struct is_known_path_source_tag :
-    public boost::true_type
+    public std::true_type
 {
 };
 
 template< >
 struct is_known_path_source_tag< unknown_type_tag > :
-    public boost::false_type
+    public std::false_type
 {
 };
 
@@ -336,7 +335,7 @@ struct is_path_source :
 //! The trait indicates whether the type is a path Source that is natively supported by path::string_type as the source for construction/assignment/appending
 template< typename T >
 struct is_native_path_source :
-    public boost::integral_constant< bool, path_source_traits< T >::is_native >
+    public std::integral_constant< bool, path_source_traits< T >::is_native >
 {
 };
 
@@ -344,19 +343,19 @@ struct is_native_path_source :
 //! The trait indicates whether the type is one of the supported path character types
 template< typename T >
 struct is_path_char_type :
-    public boost::false_type
+    public std::false_type
 {
 };
 
 template< >
 struct is_path_char_type< char > :
-    public boost::true_type
+    public std::true_type
 {
 };
 
 template< >
 struct is_path_char_type< wchar_t > :
-    public boost::true_type
+    public std::true_type
 {
 };
 
@@ -370,10 +369,13 @@ struct is_iterator_to_path_chars :
 //! The trait indicates whether the type is an iterator over a sequence of path characters
 template< typename Iterator >
 struct is_path_source_iterator :
-    public boost::conjunction<
-        boost::iterators::is_iterator< Iterator >,
-        is_iterator_to_path_chars< Iterator >
-    >::type
+    public std::integral_constant<
+        bool,
+        detail::conjunction<
+            boost::iterators::is_iterator< Iterator >,
+            is_iterator_to_path_chars< Iterator >
+        >::value
+    >
 {
 };
 
@@ -381,36 +383,36 @@ struct is_path_source_iterator :
 //! The trait indicates whether the type is a pointer to a sequence of native path characters
 template< typename T >
 struct is_native_char_ptr :
-    public boost::false_type
+    public std::false_type
 {
 };
 
 template< >
 struct is_native_char_ptr< path_native_char_type* > :
-    public boost::true_type
+    public std::true_type
 {
 };
 
 template< >
 struct is_native_char_ptr< const path_native_char_type* > :
-    public boost::true_type
+    public std::true_type
 {
 };
 
 
-//! Converts character encoding using the supplied codecvt facet. If \a cvt is \c NULL then \c path::codecvt() will be used.
+//! Converts character encoding using the supplied codecvt facet. If \a cvt is \c nullptr then \c path::codecvt() will be used.
 BOOST_FILESYSTEM_DECL
-void convert(const char* from, const char* from_end, std::wstring& to, const codecvt_type* cvt = NULL);
+void convert(const char* from, const char* from_end, std::wstring& to, const codecvt_type* cvt = nullptr);
 
 //! \overload convert
 BOOST_FILESYSTEM_DECL
-void convert(const wchar_t* from, const wchar_t* from_end, std::string& to, const codecvt_type* cvt = NULL);
+void convert(const wchar_t* from, const wchar_t* from_end, std::string& to, const codecvt_type* cvt = nullptr);
 
 
 //  Source dispatch  -----------------------------------------------------------------//
 
 template< typename Source, typename Callback >
-typename Callback::result_type dispatch(Source const& source, Callback cb, const codecvt_type* cvt = NULL);
+typename Callback::result_type dispatch(Source const& source, Callback cb, const codecvt_type* cvt = nullptr);
 
 template< typename Callback >
 BOOST_FORCEINLINE typename Callback::result_type dispatch(const char* source, Callback cb, const codecvt_type* cvt, ntcts_type_tag)
@@ -437,12 +439,12 @@ BOOST_FORCEINLINE typename Callback::result_type dispatch(Source const& source,
     return cb(src.data(), src.data() + src.size(), cvt);
 }
 
-#if !defined(BOOST_FILESYSTEM_NO_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
+#if defined(BOOST_FILESYSTEM_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
 
 template< typename Callback >
 BOOST_FORCEINLINE typename Callback::result_type dispatch(std::vector< char > const& source, Callback cb, const codecvt_type* cvt, range_type_tag)
 {
-    const char* data = NULL, *data_end = NULL;
+    const char* data = nullptr, *data_end = nullptr;
     if (!source.empty())
     {
         data = &source[0];
@@ -454,7 +456,7 @@ BOOST_FORCEINLINE typename Callback::result_type dispatch(std::vector< char > co
 template< typename Callback >
 BOOST_FORCEINLINE typename Callback::result_type dispatch(std::vector< wchar_t > const& source, Callback cb, const codecvt_type* cvt, range_type_tag)
 {
-    const wchar_t* data = NULL, *data_end = NULL;
+    const wchar_t* data = nullptr, *data_end = nullptr;
     if (!source.empty())
     {
         data = &source[0];
@@ -463,7 +465,7 @@ BOOST_FORCEINLINE typename Callback::result_type dispatch(std::vector< wchar_t >
     return cb(data, data_end, cvt);
 }
 
-#endif // !defined(BOOST_FILESYSTEM_NO_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
+#endif // defined(BOOST_FILESYSTEM_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
 
 // Defined in directory.hpp to avoid circular header dependencies
 template< typename Callback >
@@ -473,7 +475,7 @@ template< typename Source, typename Callback >
 BOOST_FORCEINLINE typename Callback::result_type dispatch(Source const& source, Callback cb, const codecvt_type* cvt)
 {
     return path_traits::dispatch(source, cb, cvt,
-        typename path_traits::path_source_traits< typename boost::remove_cv< Source >::type >::tag_type());
+        typename path_traits::path_source_traits< typename std::remove_cv< Source >::type >::tag_type());
 }
 
 
@@ -496,19 +498,35 @@ yes_type check_convertible(std::wstring_view const&);
 #endif
 yes_type check_convertible(boost::basic_string_view< char, std::char_traits< char > > const&);
 yes_type check_convertible(boost::basic_string_view< wchar_t, std::char_traits< wchar_t > > const&);
-#if !defined(BOOST_NO_CXX11_NULLPTR)
 no_type check_convertible(std::nullptr_t);
-#endif
 no_type check_convertible(...);
 
 } // namespace is_convertible_to_path_source_impl
 
-//! The type trait indicates whether the type has a conversion path to one of the path source types
+template< typename T >
+struct check_is_convertible_to_path_source :
+    public std::integral_constant<
+        bool,
+        sizeof(is_convertible_to_path_source_impl::check_convertible(std::declval< T const& >())) == sizeof(yes_type)
+    >
+{
+};
+
+/*!
+ * \brief The type trait indicates whether the type has a conversion path to one of the path source types.
+ *
+ * \note The type trait returns `false` if the type is convertible to `path`. This prevents testing other
+ *       conversion paths and forces the conversion to `path` to be chosen instead, to invoke a non-template
+ *       member of `path` accepting a `path` argument.
+ */
 template< typename T >
 struct is_convertible_to_path_source :
-    public boost::integral_constant<
+    public std::integral_constant<
         bool,
-        sizeof(is_convertible_to_path_source_impl::check_convertible(boost::declval< T const& >())) == sizeof(yes_type)
+        detail::conjunction<
+            detail::negation< std::is_convertible< T, path > >,
+            check_is_convertible_to_path_source< T >
+        >::value
     >
 {
 };
@@ -525,18 +543,16 @@ namespace is_convertible_to_std_string_view_impl {
 
 yes_type check_convertible(std::string_view const&);
 yes_type check_convertible(std::wstring_view const&);
-#if !defined(BOOST_NO_CXX11_NULLPTR)
 no_type check_convertible(std::nullptr_t);
-#endif
 no_type check_convertible(...);
 
 } // namespace is_convertible_to_std_string_view_impl
 
 template< typename T >
-struct is_convertible_to_std_string_view :
-    public boost::integral_constant<
+struct check_is_convertible_to_std_string_view :
+    public std::integral_constant<
         bool,
-        sizeof(is_convertible_to_std_string_view_impl::check_convertible(boost::declval< T const& >())) == sizeof(yes_type)
+        sizeof(is_convertible_to_std_string_view_impl::check_convertible(std::declval< T const& >())) == sizeof(yes_type)
     >
 {
 };
@@ -551,29 +567,39 @@ yes_type check_convertible(boost::container::basic_string< char, std::char_trait
 yes_type check_convertible(boost::container::basic_string< wchar_t, std::char_traits< wchar_t >, void > const&);
 yes_type check_convertible(boost::basic_string_view< char, std::char_traits< char > > const&);
 yes_type check_convertible(boost::basic_string_view< wchar_t, std::char_traits< wchar_t > > const&);
-#if !defined(BOOST_NO_CXX11_NULLPTR)
 no_type check_convertible(std::nullptr_t);
-#endif
 no_type check_convertible(...);
 
 } // namespace is_convertible_to_path_source_non_std_string_view_impl
 
 template< typename T >
-struct is_convertible_to_path_source_non_std_string_view :
-    public boost::integral_constant<
+struct check_is_convertible_to_path_source_non_std_string_view :
+    public std::integral_constant<
         bool,
-        sizeof(is_convertible_to_path_source_non_std_string_view_impl::check_convertible(boost::declval< T const& >())) == sizeof(yes_type)
+        sizeof(is_convertible_to_path_source_non_std_string_view_impl::check_convertible(std::declval< T const& >())) == sizeof(yes_type)
     >
 {
 };
 
-//! The type trait indicates whether the type has a conversion path to one of the path source types
+/*!
+ * \brief The type trait indicates whether the type has a conversion path to one of the path source types.
+ *
+ * \note The type trait returns `false` if the type is convertible to `path`. This prevents testing other
+ *       conversion paths and forces the conversion to `path` to be chosen instead, to invoke a non-template
+ *       member of `path` accepting a `path` argument.
+ */
 template< typename T >
 struct is_convertible_to_path_source :
-    public boost::disjunction<
-        is_convertible_to_std_string_view< T >,
-        is_convertible_to_path_source_non_std_string_view< T >
-    >::type
+    public std::integral_constant<
+        bool,
+        detail::conjunction<
+            detail::negation< std::is_convertible< T, path > >,
+            detail::disjunction<
+                check_is_convertible_to_std_string_view< T >,
+                check_is_convertible_to_path_source_non_std_string_view< T >
+            >
+        >::value
+    >
 {
 };
 
@@ -683,9 +709,9 @@ BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl(std::
 #endif // !defined(BOOST_NO_CXX17_HDR_STRING_VIEW)
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = NULL)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = nullptr)
 {
-    typedef typename boost::remove_cv< Source >::type source_t;
+    typedef typename std::remove_cv< Source >::type source_t;
     return path_traits::dispatch_convertible_impl< source_t >(source, cb, cvt);
 }
 
@@ -706,22 +732,22 @@ BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_sv_impl(st
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE typename boost::disable_if_c<
-    is_convertible_to_std_string_view< typename boost::remove_cv< Source >::type >::value,
+BOOST_FORCEINLINE typename std::enable_if<
+    !check_is_convertible_to_std_string_view< typename std::remove_cv< Source >::type >::value,
     typename Callback::result_type
->::type dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = NULL)
+>::type dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = nullptr)
 {
-    typedef typename boost::remove_cv< Source >::type source_t;
+    typedef typename std::remove_cv< Source >::type source_t;
     return path_traits::dispatch_convertible_impl< source_t >(source, cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE typename boost::enable_if_c<
-    is_convertible_to_std_string_view< typename boost::remove_cv< Source >::type >::value,
+BOOST_FORCEINLINE typename std::enable_if<
+    check_is_convertible_to_std_string_view< typename std::remove_cv< Source >::type >::value,
     typename Callback::result_type
->::type dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = NULL)
+>::type dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = nullptr)
 {
-    typedef typename boost::remove_cv< Source >::type source_t;
+    typedef typename std::remove_cv< Source >::type source_t;
     return path_traits::dispatch_convertible_sv_impl< source_t >(source, cb, cvt);
 }
 
