@@ -1,6 +1,7 @@
 // Boost.Geometry
 
-// Copyright (c) 2022, Oracle and/or its affiliates.
+// Copyright (c) 2022-2024, Oracle and/or its affiliates.
+// Contributed and/or modified by Vissarion Fysikopoulos, on behalf of Oracle
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
 // Licensed under the Boost Software License version 1.0.
@@ -216,22 +217,12 @@ private:
     template <typename Out, typename Strategy, std::enable_if_t<! util::is_pointlike<Out>::value, int> = 0>
     static void merge_two(Out const& g1, Out const& g2, Out& out, Strategy const& strategy)
     {
-        using rescale_policy_type = typename geometry::rescale_overlay_policy_type
-            <
-                Out, Out, typename Strategy::cs_tag
-            >::type;
-
-        rescale_policy_type robust_policy
-            = geometry::get_rescale_policy<rescale_policy_type>(
-                    g1, g2, strategy);
-
         geometry::dispatch::intersection_insert
             <
                 Out, Out, typename boost::range_value<Out>::type,
                 overlay_union
             >::apply(g1,
                      g2,
-                     robust_policy,
                      geometry::range::back_inserter(out),
                      strategy);
     }
@@ -244,7 +235,6 @@ private:
                 Out, Out, typename boost::range_value<Out>::type
             >::apply(g1,
                      g2,
-                     0, // dummy robust policy
                      geometry::range::back_inserter(out),
                      strategy);
     }
