@@ -1,12 +1,17 @@
 #ifndef BOOST_LEAF_CONFIG_TLS_FREERTOS_HPP_INCLUDED
 #define BOOST_LEAF_CONFIG_TLS_FREERTOS_HPP_INCLUDED
 
-// Copyright 2018-2023 Emil Dotchevski and Reverge Studios, Inc.
-// Copyright (c) 2022 Khalil Estell
-
+// Copyright 2018-2024 Emil Dotchevski and Reverge Studios, Inc.
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
+// Copyright (c) 2022 Khalil Estell
+
+// LEAF requires thread local storage support for pointers and for uin32_t values.
+
+// This header implements "thread local" storage via FreeTOS functions
+// pvTaskGetThreadLocalStoragePointer / pvTaskSetThreadLocalStoragePointer
+
 #include <task.h>
 
 #ifndef BOOST_LEAF_USE_TLS_ARRAY
@@ -39,4 +44,4 @@ namespace tls
 
 } }
 
-#endif
+#endif // BOOST_LEAF_CONFIG_TLS_FREERTOS_HPP_INCLUDED
