@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2023 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
+// Copyright (c) 2019-2024 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -8,13 +8,22 @@
 #ifndef BOOST_MYSQL_BUFFER_PARAMS_HPP
 #define BOOST_MYSQL_BUFFER_PARAMS_HPP
 
+#include <boost/mysql/defaults.hpp>
+
+#include <boost/config.hpp>
+
 #include <cstddef>
 
 namespace boost {
 namespace mysql {
 
 /**
- * \brief Buffer configuration parameters for a connection.
+ * \brief (Legacy) Buffer configuration parameters for a connection.
+ *
+ * \par Legacy
+ * This class is used with the legacy \ref connection class.
+ * New code should use \ref any_connection, instead.
+ * The equivalent to `buffer_params` is \ref any_connection_params::initial_buffer_size.
  */
 class buffer_params
 {
@@ -22,7 +31,7 @@ class buffer_params
 
 public:
     /// The default value of \ref initial_read_size.
-    static constexpr std::size_t default_initial_read_size = 1024;
+    static BOOST_INLINE_CONSTEXPR std::size_t default_initial_read_size = default_initial_read_buffer_size;
 
     /**
      * \brief Initializing constructor.
