@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2023 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
+// Copyright (c) 2019-2024 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -25,6 +25,9 @@
 #include <boost/mysql/detail/typing/pos_map.hpp>
 #include <boost/mysql/detail/void_t.hpp>
 
+#include <boost/mp11/algorithm.hpp>
+#include <boost/mp11/utility.hpp>
+
 #include <cstdint>
 #include <limits>
 #include <string>
@@ -97,7 +100,7 @@ template <class T>
 struct int_traits<T, true, 1>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "int8_t";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "int8_t";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -113,7 +116,7 @@ template <class T>
 struct int_traits<T, false, 1>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "uint8_t";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "uint8_t";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -129,7 +132,7 @@ template <class T>
 struct int_traits<T, true, 2>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "int16_t";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "int16_t";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -147,7 +150,7 @@ template <class T>
 struct int_traits<T, false, 2>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "uint16_t";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "uint16_t";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -165,7 +168,7 @@ template <class T>
 struct int_traits<T, true, 4>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "int32_t";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "int32_t";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -185,7 +188,7 @@ template <class T>
 struct int_traits<T, false, 4>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "uint32_t";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "uint32_t";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -205,7 +208,7 @@ template <class T>
 struct int_traits<T, true, 8>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "int64_t";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "int64_t";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -226,7 +229,7 @@ template <class T>
 struct int_traits<T, false, 8>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "uint64_t";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "uint64_t";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -313,7 +316,7 @@ template <>
 struct readable_field_traits<bool, void>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "bool";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "bool";
     static bool meta_check(meta_check_context& ctx)
     {
         return ctx.current_meta().type() == column_type::tinyint && !ctx.current_meta().is_unsigned();
@@ -333,7 +336,7 @@ template <>
 struct readable_field_traits<float, void>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "float";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "float";
     static bool meta_check(meta_check_context& ctx)
     {
         return ctx.current_meta().type() == column_type::float_;
@@ -353,7 +356,7 @@ template <>
 struct readable_field_traits<double, void>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "double";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "double";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -387,7 +390,7 @@ template <class Allocator>
 struct readable_field_traits<std::basic_string<char, std::char_traits<char>, Allocator>, void>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "string";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "string";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -420,7 +423,7 @@ template <class Allocator>
 struct readable_field_traits<std::vector<unsigned char, Allocator>, void>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "blob";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "blob";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -449,7 +452,7 @@ template <>
 struct readable_field_traits<date, void>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "date";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "date";
     static bool meta_check(meta_check_context& ctx) { return ctx.current_meta().type() == column_type::date; }
     static error_code parse(field_view input, date& output)
     {
@@ -466,7 +469,7 @@ template <>
 struct readable_field_traits<datetime, void>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "datetime";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "datetime";
     static bool meta_check(meta_check_context& ctx)
     {
         switch (ctx.current_meta().type())
@@ -491,7 +494,7 @@ template <>
 struct readable_field_traits<time, void>
 {
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = "time";
+    static BOOST_INLINE_CONSTEXPR const char* type_name = "time";
     static bool meta_check(meta_check_context& ctx) { return ctx.current_meta().type() == column_type::time; }
     static error_code parse(field_view input, time& output)
     {
@@ -530,7 +533,7 @@ struct readable_field_traits<
 {
     using value_type = typename T::value_type;
     static constexpr bool is_supported = true;
-    static constexpr const char* type_name = readable_field_traits<value_type>::type_name;
+    static BOOST_INLINE_CONSTEXPR const char* type_name = readable_field_traits<value_type>::type_name;
     static bool meta_check(meta_check_context& ctx)
     {
         ctx.set_nullability_checked();
@@ -591,30 +594,6 @@ void meta_check_field(meta_check_context& ctx)
     ctx.advance();
 }
 
-struct meta_check_field_fn
-{
-    meta_check_context ctx;
-
-    template <class T>
-    void operator()(T)
-    {
-        meta_check_field<typename T::type>(ctx);
-    }
-};
-
-template <typename ReadableFieldList>
-error_code meta_check_field_type_list(
-    span<const std::size_t> field_map,
-    name_table_t name_table,
-    metadata_collection_view meta,
-    diagnostics& diag
-)
-{
-    meta_check_field_fn fn{meta_check_context(field_map, name_table, meta)};
-    boost::mp11::mp_for_each<ReadableFieldList>(fn);
-    return fn.ctx.check_errors(diag);
-}
-
 }  // namespace detail
 }  // namespace mysql
 }  // namespace boost
